import utilities as utilities
from MLOPE import MLOPE
import time


class RunMLOPE:
    def __init__(self, train_file, settings, model_folder, test_data, path_dictionary):
        self.train_file = train_file
        self.settings = settings
        self.model_folder = model_folder
        self.test_data = test_data
        self.path_dictionary = path_dictionary

    def run(self):
        # Initialize the algorithm
        print('initialize the algorithm ...')
        ml_ope = MLOPE(self.settings['num_terms'], self.settings['num_topics'], self.settings['alpha'],
                       self.settings['tau0'], self.settings['kappa'], self.settings['iter_infer'])

        # Start
        print('-->START<--')
        i = 0
        time_start = time.time()
        while i < self.settings['iter_train']:
            i += 1
            print('\n***iter_train:%d***\n' % (i))
            datafp = open(self.train_file, 'r')
            j = 0
            while True:
                j += 1
                (wordids, wordcts) = utilities.read_minibatch_list_frequencies(datafp, self.settings['batch_size'])
                # Stop condition
                if len(wordids) == 0:
                    break

                print('---num_minibatch:%d---' % (j))
                ml_ope.static_online(wordids, wordcts)

            datafp.close()

            # write perplexity
            print('compute perplexity')
            test_LD2 = utilities.compute_perplexities_vb(ml_ope.beta, self.settings['alpha'], self.settings['eta'],
                                                    self.settings['iter_infer'], self.test_data)
            file_name_per = '%s/perplexities.csv' % self.model_folder
            utilities.write_perplexities(test_LD2, file_name_per, i)

        # write settings
        print('write setting ...')
        file_name_setting = '%s/setting.txt' % self.model_folder
        utilities.write_setting(self.settings, file_name_setting)

        # write final model to file
        print('write model ...')
        file_name_beta = '%s/beta_final.dat' % self.model_folder
        utilities.write_topics(ml_ope.beta, file_name_beta)
        time_end = time.time()

        # write topic
        file_name_topic = '%s/topic.txt' % self.model_folder
        dictionary = utilities.read_dictionary(self.path_dictionary)
        utilities.write_topics2(file_name_beta, file_name_topic, dictionary, num_topic=self.settings['num_topics'])
        # Finish
        print('-------------------------DONE-------------------------', (time_end - time_start))



import sys
import numpy as np
import per_vb as per_vb
import csv
"""
    Read all documents in the file and stores terms and counts in lists.
"""


def read_data(filename):
    wordids = list()
    wordcts = list()
    fp = open(filename, 'r')
    while True:
        line = fp.readline()
        # check end of file
        if len(line) < 1:
            break
        terms = line.split()
        doc_length = int(terms[0])
        ids = np.zeros(doc_length, dtype=np.int32)
        cts = np.zeros(doc_length, dtype=np.int32)
        for j in range(1, doc_length + 1):
            term_count = terms[j].split(':')
            ids[j - 1] = int(term_count[0])
            cts[j - 1] = int(term_count[1])
        wordids.append(ids)
        wordcts.append(cts)
    fp.close()
    return wordids, wordcts


# read mini batch
def read_minibatch_list_frequencies(fp, batch_size):
    wordids = list()
    wordcts = list()
    for i in range(batch_size):
        line = fp.readline()
        # check end of file
        if len(line) < 5:
            break
        terms = line.split()
        doc_length = int(terms[0])
        ids = np.zeros(doc_length, dtype=np.int32)
        cts = np.zeros(doc_length, dtype=np.int32)
        for j in range(1, doc_length + 1):
            term_count = terms[j].split(':')
            ids[j - 1] = int(term_count[0])
            cts[j - 1] = int(term_count[1])
        wordids.append(ids)
        wordcts.append(cts)
    return wordids, wordcts


"""
    Read data for computing perplexities.
"""


def read_data_for_perpl(test_data_folder):
    filename_part1 = '%s/data_test_part_1.txt' % (test_data_folder)
    filename_part2 = '%s/data_test_part_2.txt' % (test_data_folder)
    (wordids_1, wordcts_1) = read_data(filename_part1)
    (wordids_2, wordcts_2) = read_data(filename_part2)
    data_test = list()
    data_test.append(wordids_1)
    data_test.append(wordcts_1)
    data_test.append(wordids_2)
    data_test.append(wordcts_2)
    return data_test


def read_train_data_for_perpl(train_data_folder):
    filename_part1 = '%s/data_train_part_1.txt' % (train_data_folder)
    filename_part2 = '%s/data_train_part_2.txt' % (train_data_folder)
    (wordids_1, wordcts_1) = read_data(filename_part1)
    (wordids_2, wordcts_2) = read_data(filename_part2)
    data_test = list()
    data_test.append(wordids_1)
    data_test.append(wordcts_1)
    data_test.append(wordids_2)
    data_test.append(wordcts_2)
    return data_test


def read_setting(file_name):
    f = open(file_name, 'r')
    settings = f.readlines()
    f.close()
    sets = list()
    vals = list()
    for i in range(len(settings)):
        # print'%s\n'%(settings[i])
        if settings[i][0] == '#':
            continue
        set_val = settings[i].split(':')
        sets.append(set_val[0])
        vals.append(float(set_val[1]))
    ddict = dict(zip(sets, vals))
    ddict['num_terms'] = int(ddict['num_terms'])
    ddict['num_topics'] = int(ddict['num_topics'])
    ddict['iter_train'] = int(ddict['iter_train'])
    ddict['iter_infer'] = int(ddict['iter_infer'])
    ddict['batch_size'] = int(ddict['batch_size'])
    return ddict


# --------------------------------------------------------------------------------
"""
    Compute document sparsity.
"""


def compute_sparsity(doc_tp, batch_size, num_topics, _type):
    sparsity = np.zeros(batch_size, dtype=np.float)
    if _type == 'z':
        for d in range(batch_size):
            N_z = np.zeros(num_topics, dtype=np.int)
            N = len(doc_tp[d])
            for i in range(N):
                N_z[doc_tp[d][i]] += 1
            sparsity[d] = len(np.where(N_z != 0)[0])

    else:
        for d in range(batch_size):
            sparsity[d] = len(np.where(doc_tp[d] > 1e-10)[0])
        sparsity /= num_topics

    return np.mean(sparsity)


"""
    Compute perplexities, employing Variational Bayes.
"""


def compute_perplexities_vb(beta, alpha, eta, max_iter, data_test):
    vb = per_vb.VB(beta, alpha, eta, max_iter)
    LD2 = vb.compute_perplexity(data_test)
    return LD2


"""
    Create list of top words of topics.
"""


def list_top(beta, tops):
    min_float = -sys.float_info.max
    num_tops = beta.shape[0]
    list_tops = list()
    for k in range(num_tops):
        top = list()
        arr = np.array(beta[k, :], copy=True)
        for t in range(tops):
            index = arr.argmax()
            top.append(index)
            arr[index] = min_float
        list_tops.append(top)
    return list_tops


# --------------------------------------------------------------------------------
def write_topics(beta, file_name):
    num_terms = beta.shape[1]
    num_topics = beta.shape[0]
    f = open(file_name, 'w')
    for k in range(num_topics):
        # f.write('topic ' + str(k) + ': ')
        for i in range(num_terms - 1):
            f.write('%.10f ' % (beta[k][i]))
        f.write('%.10f\n' % (beta[k][num_terms - 1]))
    f.close()


def write_perplexities(test_LD2, file_name, i):
    with open(file_name, 'a') as csvfile:
        writer = csv.writer(csvfile, quotechar='\t', quoting=csv.QUOTE_MINIMAL)
        writer.writerow([i, test_LD2])


def write_time(i, j, time_e, time_m, file_name):
    f = open(file_name, 'a')
    f.write('tloop_%d_iloop_%d, %f, %f, %f,\n' % (i, j, time_e, time_m, time_e + time_m))
    f.close()


def write_loop(i, j, file_name):
    f = open(file_name, 'w')
    f.write('%d, %d' % (i, j))
    f.close()


def write_file(i, j, beta, time_e, time_m, theta, sparsity, test_LD2, train_LDA2, list_tops, tops, model_folder):
    # per_file_name = '%s/perplexities_%d.csv' % (model_folder, i)
    per_file_name = '%s/perplexities.csv' % model_folder
    time_file_name = '%s/time_%d.csv' % (model_folder, i)
    loop_file_name = '%s/loops.csv' % model_folder
    # write perplexities
    write_perplexities(test_LD2, train_LDA2, per_file_name, i)
    # write time
    write_time(i, j, time_e, time_m, time_file_name)
    # write loop
    write_loop(i, j, loop_file_name)


def write_setting(ddict, file_name):
    keys = list(ddict.keys())
    vals = list(ddict.values())
    f = open(file_name, 'w')
    for i in range(len(keys)):
        f.write('%s: %f\n' % (keys[i], vals[i]))
    f.close()


# --------------------------------------------------------------------------------
# my: read dictionary
def read_dictionary(file_name):
    dict = {}
    with open(file_name, encoding='utf8') as file:
        for line in file:
            item = line.split(':')
            dict[item[0]] = item[1].replace('\n', '')
    return dict


def read_dictionary2(file_name):
    dictionary = {}
    i = 0
    with open(file_name, encoding='utf8') as f:
        for line in f:
            dictionary[i] = line.replace('\n', '')
            i += 1
    return dictionary


# my: show topic
def show_topics(path_file_beta, dictionary, num_topic=10):
    # print(dictionary)
    num_term = len(dictionary)
    with open(path_file_beta, 'r') as f:
        for i in range(num_topic):
            line = f.readline().split()
            item = {}
            for j in range(len(line)):
                if j >= num_term:
                    break
                item[line[j]] = dictionary[str(j)]
                # item[line[j]] = dictionary[j]
            a = sorted(zip(item.keys(), item.values()), reverse=True)[:100]
            print('topic ', i)
            for w, t in a:
                print(w, ': ', t)


def write_topics2(path_file_beta, path_out, dictionary, num_topic=10):
    # print(dictionary)
    num_term = len(dictionary)
    file_topic_out = open(path_out, 'w', encoding='utf8')
    with open(path_file_beta, 'r') as f:
        for i in range(num_topic):
            line = f.readline().split()
            # print(len(line))
            item = {}
            for j in range(len(line)):
                if j >= num_term:
                    break
                item[line[j]] = dictionary[str(j)]
                # item[line[j]] = dictionary[j]
            a = sorted(zip(item.keys(), item.values()), reverse=True)[:100]
            file_topic_out.write('topic ' + str(i) + '\n: ')
            for w, t in a:
                file_topic_out.write('\t' + str(w) + ': ' + str(t) + '\n ')


def read_file_beta(file_keyword):
    # print('a' in dict1)
    out = open('abc.txt', 'w', encoding='utf8')
    with open(file_keyword, encoding='utf8') as file:
        for topic in file:
            topic = topic.replace('\n', '')
            if topic.find('#') != 0:
                for kw in topic.split(':')[2:]:
                    if kw != '':
                        out.write(kw + ',')
                out.write('\n')
    out.close()


def init_beta(file_name, file_dict, num_topics, num_terms, eta):
    # dict1 = read_dictionary2(file_dict)
    dict1 = read_dictionary(file_dict)
    dict1 = dict(zip(dict1.values(), dict1.keys()))
    topics = {}

    i = 0
    with open(file_name, encoding='utf8') as file:
        for topic in file:
            kws = topic.split(',')
            kws_id = list()
            for kw in kws:
                if kw in dict1:
                    kws_id.append(int(dict1[kw]))
            topics[i] = kws_id
            i += 1
    # print(topics)
    define_kws = {}
    for i in topics:
        for kw in topics[i]:
            if define_kws.get(kw) is None:
                list_topics = list()
            else:
                list_topics = define_kws.get(kw)
            if i not in list_topics:
                list_topics.append(i)
            define_kws[kw] = list_topics

    # print(define_kws)
    sstats = np.random.gamma(100., 1. / 100., (num_topics, num_terms))
    for wid, t in define_kws.items():
        sstats[:, wid] = np.random.gamma(0.1, 0.05, num_topics)

    for wid, topics in define_kws.items():
        score = 10 * num_topics
        if len(topics) > 1:
            score = 4 * num_topics
        sstats[topics, wid] = score

    m_lambda = eta + sstats
    beta_sum = m_lambda.sum(axis=1)
    beta = m_lambda / beta_sum[:, np.newaxis]
    # print(beta)

    for i in beta:
        print(i.max())

    return beta


def create_define_topic(file_name):
    define_kws = {}
    i = 0
    with open(file_name, encoding='utf8') as file:
        for topic in file:
            kws = topic.split(',')
            for kw in kws:
                if define_kws.get(kw) is None:
                    list_topics = list()
                else:
                    list_topics = define_kws.get(kw)
                if i not in list_topics:
                    list_topics.append(i)
                define_kws[kw] = list_topics
            i += 1
    return define_kws


# read memory usage
# def memory_usage_ps():
#     out = subprocess.Popen(['ps', 'v', '-p', str(os.getpid())],
#     stdout=subprocess.PIPE).communicate()[0].split(b'\n')
#     vsz_index = out[0].split().index(b'RSS')
#     mem = float(out[1].split()[vsz_index])
#     return mem


import utilities
import os
import numpy as np


def load_model(path_model):
    # read setting
    path_setting_file = path_model + '/setting.txt'
    if not os.path.exists(path_setting_file):
        print('setting file not found!!!')
    else:
        setting = utilities.read_setting(path_setting_file)
    # print(setting)

    # read_beta
    beta = np.zeros((setting['num_topics'], setting['num_terms']))
    path_beta_file = path_model + '/beta_final.dat'
    if not os.path.exists(path_beta_file):
        print('beta file not found!!!')
    else:
        i = 0
        with open(path_beta_file, 'r') as file:
            for topic in file:
                t = np.array(list(map(float, topic.split())))
                beta[i] = t
                i += 1
                # print(t)
        # print(beta)
    return beta, setting['alpha'], setting['iter_infer']

def read_doc(data):
    doc = data.split()
    num_terms = int(doc[0])
    ids = np.zeros(num_terms, dtype=np.int32)
    cts = np.zeros(num_terms, dtype=np.int32)
    for i in range(1, num_terms + 1):
        term = doc[i].split(':')
        ids[i - 1] = int(term[0])
        cts[i - 1] = int(term[1])
    return ids, cts


# def infer_doc(alphad, betad, iter_infer, ids, cts, weight):
def infer_doc(model, data, weight):
    # load model, data
    ids, cts = read_doc(data)
    alphad = model[0]
    betad = model[1]
    iter_infer = model[2]
    # locate cache memory
    # locate cache memory
    beta = betad[:, ids]
    # Initialize theta randomly
    theta = np.random.rand(betad.shape[0]) + 1
    theta /= sum(theta)

    x = np.dot(theta, beta)
    # Loop
    T = [1, 0]
    for l in range(1, iter_infer):
        # Pick fi uniformly
        T[np.random.randint(2)] += 1
        # Select a vertex with the largest value of
        # derivative of the function F
        df = T[0] * np.dot(beta, cts / x) + T[1] * (alphad - 1) / theta
        index = np.argmax(df)
        alpha = 1.0 / (l + 1)
        # Update theta
        theta *= 1 - alpha
        theta[index] += alpha
        # Update x
        x = x+alpha * (beta[index, :] - x)

    dict = {}
    i = 0
    for p in theta:
        if p > weight:
            dict[i] = p
        i += 1

    # sort_dict = sorted(dict.items(), key=operator.itemgetter(1), reverse=True)

    return dict


import time
import numpy as np
import utilities


# Training model
def train(train_file, settings, model_folder, path_dictionary):
    print('initialize the algorithm ...')
    ml_ope = MLOPE(settings['num_terms'], settings['num_topics'], settings['alpha'],
                   settings['tau0'], settings['kappa'], settings['iter_infer'])

    # Start
    print('-->START<--')
    i = 0
    time_start = time.time()
    while i < settings['iter_train']:
        i += 1
        print('\n***iter_train:%d***\n' % (i))
        datafp = open(train_file, 'r')
        j = 0
        while True:
            j += 1
            (wordids, wordcts) = utilities.read_minibatch_list_frequencies(datafp, settings['batch_size'])
            # Stop condition
            if len(wordids) == 0:
                break

            print('---num_minibatch:%d---' % (j))
            ml_ope.static_online(wordids, wordcts)

        datafp.close()

    # write settings
    print('write setting ...')
    file_name_setting = '%s/setting.txt' % model_folder
    utilities.write_setting(settings, file_name_setting)

    # write final model to file
    print('write model ...')
    file_name_beta = '%s/beta_final.dat' % model_folder
    utilities.write_topics(ml_ope.beta, file_name_beta)
    time_end = time.time()

    # write topic
    file_name_topic = '%s/topic.txt' % model_folder
    dictionary = utilities.read_dictionary(path_dictionary)
    utilities.write_topics2(file_name_beta, file_name_topic, dictionary, num_topic=settings['num_topics'])
    # Finish
    print('-------------------------DONE-------------------------', (time_end - time_start))


class MLOPE:
    def __init__(self, num_terms, num_topics, alpha, tau0, kappa, iter_infer):
        """
        Arguments:
            num_terms: Number of unique terms in the corpus (length of the vocabulary).
            num_topics: Number of topics shared by the whole corpus.
            alpha: Hyperparameter for prior on topic mixture theta.
            tau0: A (positive) learning parameter that downweights early iterations.
            kappa: Learning rate: exponential decay rate should be between
                   (0.5, 1.0] to guarantee asymptotic convergence.
            iter_infer: Number of iterations of FW algorithm

        Note that if you pass the same set of all documents in the corpus every time and
        set kappa=0 this class can also be used to do batch OPE.
        """
        self.num_terms = num_terms
        self.num_topics = num_topics
        self.alpha = alpha
        self.tau0 = tau0
        self.kappa = kappa
        self.INF_MAX_ITER = iter_infer
        self.updatect = 1

        # Initializable beta (topics)
        self.beta = np.random.rand(self.num_topics, self.num_terms) + 1e-10
        # import my_MLOPE.utilities as utilities
        # self.beta = utilities.init_beta('F:\\Data\\Important Data\\3.11.2016\\define_kws.txt', 'F:\\Data\\Important Data\\3.11.2016\\dictionary3.txt', self.num_topics, self.num_terms, 0.01)

        # print(self.beta[0], '\n')
        beta_norm = self.beta.sum(axis=1)
        # print(beta_norm)
        self.beta /= beta_norm[:, np.newaxis]
        # print(self.beta[0])

    def static_online(self, wordids, wordcts):
        """
        First does an E step on the mini-batch given in wordids and
        wordcts, then uses the result of that E step to update the
        topics in M step.

        Arguments:
        batch_size: Number of documents of the mini-batch.
        wordids: A list whose each element is an array (terms), corresponding to a document.
                 Each element of the array is index of a unique term, which appears in the document,
                 in the vocabulary.
        wordcts: A list whose each element is an array (frequency), corresponding to a document.
                 Each element of the array says how many time the corresponding term in wordids appears
                 in the document.
        Returns time the E and M steps have taken and the list of topic mixtures of all documents in the mini-batch.
        """
        batch_size = len(wordids)
        # E_step
        start1 = time.time()
        theta = self.e_step(batch_size, wordids, wordcts)
        end1 = time.time()

        # M_step
        start2 = time.time()
        self.m_step(batch_size, wordids, wordcts, theta)
        end2 = time.time()
        return end1 - start1, end2 - start2, theta

    def e_step(self, batch_size, wordids, wordcts):
        """
        Does e step

        Returns topic mixtures theta.
        """
        # Declare theta of minibatch
        theta = np.zeros((batch_size, self.num_topics))
        # Inference
        for d in range(batch_size):
            thetad = self.infer_doc(wordids[d], wordcts[d])
            theta[d, :] = thetad

        return (theta)

    def infer_doc(self, ids, cts):
        """
        Does inference for a document using Online MAP Estimation algorithm.

        Arguments:
        ids: an element of wordids, corresponding to a document.
        cts: an element of wordcts, corresponding to a document.

        Returns inferred theta.
        """
        # locate cache memory
        beta = self.beta[:, ids]
        # Initialize theta randomly
        theta = np.random.rand(self.num_topics) + 1
        theta /= sum(theta)

        x = np.dot(theta, beta)
        # Loop
        T = [1, 0]
        for l in range(1, self.INF_MAX_ITER):
            # Pick fi uniformly
            T[np.random.randint(2)] += 1
            # Select a vertex with the largest value of
            # derivative of the function F
            df = T[0] * np.dot(beta, cts / x) + T[1] * (self.alpha - 1) / theta
            index = np.argmax(df)
            alpha = 1.0 / (l + 1)
            # Update theta
            theta *= 1 - alpha
            theta[index] += alpha
            # Update x
            x = x + alpha * (beta[index, :] - x)

        return theta

    def m_step(self, batch_size, wordids, wordcts, theta):
        """
        Does m step: update global variables beta.
        """
        # Compute intermediate beta which is denoted as "unit beta"
        beta = np.zeros((self.num_topics, self.num_terms), dtype=float)
        for d in range(batch_size):
            beta[:, wordids[d]] += np.outer(theta[d], wordcts[d])
        # Check zeros index
        beta_sum = beta.sum(axis=0)
        ids = np.where(beta_sum != 0)[0]
        unit_beta = beta[:, ids]
        # Nomalize the intermediate beta
        unit_beta_norm = unit_beta.sum(axis=1)
        unit_beta /= unit_beta_norm[:, np.newaxis]
        # Update beta
        rhot = pow(self.tau0 + self.updatect, -self.kappa)
        self.rhot = rhot
        self.beta *= (1 - rhot)
        self.beta[:, ids] += unit_beta * rhot
        self.updatect += 1


import numpy as np
from scipy.special import psi
import utilities


def dirichlet_expectation(alpha):
    """
    For a vector theta ~ Dir(alpha), computes E[log(theta)] given alpha.
    """
    if len(alpha.shape) == 1:
        return psi(alpha) - psi(np.sum(alpha))
    return psi(alpha) - psi(np.sum(alpha, 1))[:, np.newaxis]


class VB:
    """
    Compute perplexity, employing Variantional Bayes algorithm.
    """

    def __init__(self, _lambda, alpha, eta, max_iter):
        """
        Arguments:
            _Lambda: Variational parameters of topics of the learned model.
            alpha: Hyperparameter for prior on topic mixture theta.
            eta: Hyperparameter for prior on topics beta.
            max_infer: Number of iterations of FW algorithm.
        """
        self._lambda = np.copy(_lambda) + 1e-10
        self._K = _lambda.shape[0]
        self._W = _lambda.shape[1]
        self._alpha = alpha
        self._eta = eta
        self._max_iter = max_iter

        # normalize lambda
        _lambda_norm = self._lambda.sum(axis=1)
        self._lambda /= _lambda_norm[:, np.newaxis]

    def do_e_step(self, batch_size, wordids, wordcts):
        """
        Does infernce for documents in 'w_obs' part.
        Arguments:
            batch_size: number of documents to be infered.
            wordids: A list whose each element is an array (terms), corresponding to a document.
                 Each element of the array is index of a unique term, which appears in the document,
                 in the vocabulary.
            wordcts: A list whose each element is an array (frequency), corresponding to a document.
                 Each element of the array says how many time the corresponding term in wordids appears
                 in the document.
        Returns: gamma the variational parameter of topic mixture (theta).
        """
        gamma = 1 * np.random.gamma(100., 1. / 100., (batch_size, self._K))
        Elogtheta = dirichlet_expectation(gamma)
        expElogtheta = np.exp(Elogtheta)
        # Now, for each document d update that document's gamma and phi
        for d in range(0, batch_size):
            # Locate memory
            ids = wordids[d]
            cts = wordcts[d]
            gammad = gamma[d, :]
            Elogthetad = Elogtheta[d, :]
            expElogthetad = expElogtheta[d, :]
            expElogbetad = self._lambda[:, ids]
            phinorm = np.dot(expElogthetad, expElogbetad) + 1e-100
            # Iterate between gamma and phi until convergence
            for it in range(0, self._max_iter):
                print("E_Step: " + str(utilities.memory_usage_ps()))
                gammad = self._alpha + expElogthetad * \
                                       np.dot(cts / phinorm, expElogbetad.T)
                Elogthetad = dirichlet_expectation(gammad)
                expElogthetad = np.exp(Elogthetad)
                phinorm = np.dot(expElogthetad, expElogbetad) + 1e-100
            gammad /= sum(gammad)
            gamma[d, :] = gammad
        return gamma

    def compute_lkh_d2(self, gammad, wordids_2d, wordcts_2d):
        """
        Compute log predictive probability for each document in 'w_ho' part.
        """
        ld2 = 0.
        frequency = 0
        for j in range(len(wordids_2d)):
            print("compute_lkh_d2: " + str(utilities.memory_usage_ps()))
            P = np.dot(gammad, self._lambda[:, wordids_2d[j]])
            ld2 += wordcts_2d[j] * np.log(P)
            frequency += wordcts_2d[j]
        if frequency != 0:
            result = ld2 / frequency
        else:
            result = ld2
        return result

    def compute_perplexity(self, data_test):
        """
        Compute log predictive probability for all documents in 'w_ho' part.
        """
        batch_size = len(data_test[0])
        # E step
        gamma = self.do_e_step(batch_size, data_test[0], data_test[1])
        # Compute perplexity
        LD2 = 0.
        for d in range(batch_size):
            LD2 += self.compute_lkh_d2(gamma[d], data_test[2][d], data_test[3][d])
        return LD2 / batch_size



from setuptools import setup

setup(name='qlda',
      version='0.1.3',
      description='Lda-MLOPE',
      url='https://github.com/Quangbd/qlda',
      author='Quang Bui Duc',
      author_email='quangbd.hust@gmail.com',
      license='NONE',
      packages=['qlda'],
      install_requires=[
          'numpy', 'scipy',
      ],
      zip_safe=False)
